@@ -7,7 +7,9 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"io"
 	"os"
+	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -27,7 +29,7 @@ import (
 var ReadLine = func() string {
 	buf := bufio.NewReader(os.Stdin)
 	line, err := buf.ReadString('\n')
-	if err != nil {
+	if err != nil && (line == "" || err != io.EOF) {
 		fs.Fatalf(nil, "Failed to read line: %v", err)
 	}
 	return strings.TrimSpace(line)
@@ -148,10 +150,8 @@ func Choose(what string, kind string, choices, help []string, defaultValue strin
 		result := ReadLine()
 		i, err := strconv.Atoi(result)
 		if err != nil {
-			for _, v := range choices {
-				if result == v {
-					return result
-				}
+			if slices.Contains(choices, result) {
+				return result
 			}
 			if result == "" {
 				// If empty string is in the predefined list of choices it has already been returned above.
